/*
 * File:        TFT, keypad, DAC, LED, PORT EXPANDER test
 *              With serial interface to PuTTY console
 *              DMA GetMachineBuffer
 *              !!!Modified scheduler!!!
 * 
 * Author:      Bruce Land
 * For use with Sean Carroll's Big Board
 * http://people.ece.cornell.edu/land/courses/ece4760/PIC32/target_board.html
 * Target PIC:  PIC32MX250F128B
 */

////////////////////////////////////
// clock AND protoThreads configure!
// You MUST check this file!
#include "config_1_3_2.h"
// threading library
#include "pt_cornell_1_3_2.h"
// yup, the expander
#include "port_expander_brl4.h"

#include <math.h>
////////////////////////////////////
// graphics libraries
// SPI channel 1 connections to TFT
#include "tft_master.h"
#include "tft_gfx.h"
// need for rand function
#include <stdlib.h>
// need for sin function
#include <math.h>
////////////////////////////////////

// lock out timer 2 interrupt during spi communication to port expander
// This is necessary if you use the SPI2 channel in an ISR.
// The ISR below runs the DAC using SPI2
#define start_spi2_critical_section INTEnable(INT_T2, 0)
#define end_spi2_critical_section INTEnable(INT_T2, 1)

////////////////////////////////////

/* Demo code for interfacing TFT (ILI9340 controller) to PIC32
 * The library has been modified from a similar Adafruit library
 */
// Adafruit data:
/***************************************************
  This is an example sketch for the Adafruit 2.2" SPI display.
  This library works with the Adafruit 2.2" TFT Breakout w/SD card
  ----> http://www.adafruit.com/products/1480

  Check out the links above for our tutorials and wiring diagrams
  These displays use SPI to communicate, 4 or 5 pins are required to
  interface (RST is optional)
  Adafruit invests time and resources providing this open source code,
  please support Adafruit and open-source hardware by purchasing
  products from Adafruit!

  Written by Limor Fried/Ladyada for Adafruit Industries.
  MIT license, all text above must be included in any redistribution
 ****************************************************/

// string buffer
//char buffer[60];

////////////////////////////////////
// DAC ISR
// A-channel, 1x, active
#define DAC_config_chan_A 0b0011000000000000
// B-channel, 1x, active
#define DAC_config_chan_B 0b1011000000000000
// DDS constant
#define two32 4294967296.0 // 2^32 
#define Fs 100000
#define ninety 695
#define sixty 605
#define onetwenty 785

//== Timer 2 interrupt handler ===========================================
// generates a DDS sinewave at a frequency set by the serial thread
// default frequency is 400 Hz.
//
volatile unsigned int DAC_data ;// output value
volatile SpiChannel spiChn = SPI_CHANNEL2 ;	// the SPI channel to use
volatile int spiClkDiv = 4 ; // 10 MHz max speed for port expander!!
// the DDS units:
volatile unsigned int phase_accum_main, phase_incr_main=400.0*two32/Fs ;//
// DDS sine table
#define sine_table_size 256
volatile int sin_table[sine_table_size];

int sys_time_seconds ;

//initialize adc for potentiometer 
volatile int adc_9 = 100;

volatile int des_angle = 695 ;
volatile float pid_p = 0;
volatile float pid_i = 0;
volatile float pid_d = 0;
volatile int error = 0;
#define total_cards num_deal*num_players
volatile int prev_adc = -1.5;
volatile int last_error = 0;

volatile float prop = 0;
volatile float diff = 0;
volatile float integ = 0;
volatile float v_control = 0;
volatile int deal_all = 0;
volatile int num = 0;
////////////////////////////////////

#define generate_period 1
#define dc 17000
#define s_dc 25000
#define time_for_all 10000
#define time_for_half 6000
#define time_for_sout 3000

#define time_for_360_CCW 2500
#define time_for_360_CW 6450
#define dc_CCW 10000
#define dc_CW 32000

volatile int deal_dc = dc;
volatile int time_for_one = 250;

int x = 0;
volatile int motor_disp = 0;
volatile int d_motor_disp = 0;

volatile int dutycycle = 0;

volatile int num_shuffle = 0;
volatile int ns = 0;
volatile int deal_start = 0;
volatile int restart = 0;
volatile int num_deal = 0;
volatile int num_players = 0;

volatile int d_dutycycle = 0;
volatile int d2_dutycycle = 0;
volatile int r_dutycycle = 0;

volatile int ask_players = 1;
//volatile int ask_shuffle = 0;
volatile int ask_num_deal = 0;
volatile int pressed = 1;

volatile int curr_player = 0;
volatile int curr_card = 0;
volatile int prev_player = 0;
int player_arr[52] = {};

////////////////////////////////////

void __ISR(_TIMER_2_VECTOR, ipl2) Timer2Handler(void) {
    //int error;
    //int junk;
    if (total_cards > 52){
        deal_all = 1;
    }
    mT2ClearIntFlag();
    
    adc_9 = ReadADC10(0);
    AcquireADC10();
    
    // shuffler
    SetDCOC1PWM(dutycycle);
    // dealer
    SetDCOC2PWM(d_dutycycle);
    // dealer
    SetDCOC3PWM(d2_dutycycle);
    // rotation
    SetDCOC4PWM(r_dutycycle);
    

}

// === print a line on TFT =====================================================
// Utilities to print a line on the TFT
// Predefined colors definitions (from tft_master.h)
//#define	ILI9340_BLACK   0x0000
//#define	ILI9340_BLUE    0x001F
//#define	ILI9340_RED     0xF800
//#define	ILI9340_GREEN   0x07E0
//#define ILI9340_CYAN    0x07FF
//#define ILI9340_MAGENTA 0xF81F
//#define ILI9340_YELLOW  0xFFE0
//#define ILI9340_WHITE   0xFFFF

// string buffer
char buffer[60];
void printLine(int line_number, char* print_buffer, short text_color, short back_color){
    // line number 0 to 31 
    /// !!! assumes tft_setRotation(0);
    // print_buffer is the string to print
    int v_pos;
    v_pos = line_number * 10 ;
    // erase the pixels
    tft_fillRoundRect(0, v_pos, 239, 8, 1, back_color);// x,y,w,h,radius,color
    tft_setTextColor(text_color); 
    tft_setCursor(0, v_pos);
    tft_setTextSize(1);
    tft_writeString(print_buffer);
}

void printLine2(int line_number, char* print_buffer, short text_color, short back_color){
    // line number 0 to 31 
    /// !!! assumes tft_setRotation(0);
    // print_buffer is the string to print
    int v_pos;
    v_pos = line_number * 20 ;
    // erase the pixels
    tft_fillRoundRect(0, v_pos, 239, 16, 1, back_color);// x,y,w,h,radius,color
    tft_setTextColor(text_color); 
    tft_setCursor(0, v_pos);
    tft_setTextSize(2);
    tft_writeString(print_buffer);
}


// === Keypad Thread =============================================
// Port Expander connections:
// y0 -- row 1 -- thru 300 ohm resistor -- avoid short when two buttons pushed
// y1 -- row 2 -- thru 300 ohm resistor
// y2 -- row 3 -- thru 300 ohm resistor
// y3 -- row 4 -- thru 300 ohm resistor
// y4 -- col 1 -- internal pullup resistor -- avoid open circuit input when no button pushed
// y5 -- col 2 -- internal pullup resistor
// y6 -- col 3 -- internal pullup resistor
// y7 -- shift key connection -- internal pullup resistor

static PT_THREAD (protothread_key(struct pt *pt))
{
    PT_BEGIN(pt);
    static int keypad, i, pattern;
    static int db_state = 0; //debouncing state
    // order is 0 thru 9 then * ==10 and # ==11
    // no press = -1
    // table is decoded to natural digit order (except for * and #)
    // with shift key codes for each key
    // keys 0-9 return the digit number
    // keys 10 and 11 are * adn # respectively
    // Keys 12 to 21 are the shifted digits
    // keys 22 and 23 are shifted * and # respectively
    static int keytable[24]=
    //        0     1      2    3     4     5     6      7    8     9    10-*  11-#
            {0xd7, 0xbe, 0xde, 0xee, 0xbd, 0xdd, 0xed, 0xbb, 0xdb, 0xeb, 0xb7, 0xe7,
    //        s0     s1    s2  s3    s4    s5    s6     s7   s8    s9    s10-* s11-#
             0x57, 0x3e, 0x5e, 0x6e, 0x3d, 0x5d, 0x6d, 0x3b, 0x5b, 0x6b, 0x37, 0x67};
    // bit pattern for each row of the keypad scan -- active LOW
    // bit zero low is first entry
    static char out_table[4] = {0b1110, 0b1101, 0b1011, 0b0111};
    // init the port expander
    start_spi2_critical_section;
    initPE();
    // PortY on Expander ports as digital outputs
    mPortYSetPinsOut(BIT_0 | BIT_1 | BIT_2 | BIT_3);    //Set port as output
    // PortY as inputs
    // note that bit 7 will be shift key input, 
    // separate from keypad
    mPortYSetPinsIn(BIT_4 | BIT_5 | BIT_6 | BIT_7);    //Set port as input
    mPortYEnablePullUp(BIT_4 | BIT_5 | BIT_6 | BIT_7);
    
    // init the Z port to try other output functions
    mPortZSetPinsOut(BIT_0 | BIT_1);    //Set port as output
    end_spi2_critical_section ;
    
    // the read-pattern if no button is pulled down by an output
    #define no_button (0x70)

      while(1) {
        // yield time
        PT_YIELD_TIME_msec(30);
    
        for (i=0; i<4; i++) {
            start_spi2_critical_section;
            // scan each rwo active-low
            writePE(GPIOY, out_table[i]);
            //reading the port also reads the outputs
            keypad  = readPE(GPIOY);
            end_spi2_critical_section;
            // was there a keypress?
            if((keypad & no_button) != no_button) { break;}
        }
        i = -1;
                // DEBOUNCING
        switch(db_state){
            case 0:               // No Press
                if((keypad & no_button) != no_button){
                    db_state = 1;
                    keypad = 0;
                }
                else{
                    db_state = 0;
                    keypad = 0;
                }
            case 1:              // Maybe Press
                if((keypad & no_button) != no_button){
                    db_state = 2;
                }
                else{
                    db_state = 0;
                    keypad = 0;
                }
            case 2:              // Press
                if((keypad & no_button) != no_button){
                    db_state = 2;
                }
                else{
                    db_state = 3;
                }
            case 3:             //Maybe No Press
                if((keypad & no_button) != no_button){
                    db_state = 2;
                }
                else{
                    db_state = 0;
                    keypad = 0;
                }
        }
        // search for keycode
        if (keypad > 0 & pressed){ // then button is pushed
            pressed = 0;
            for (i=0; i<24; i++){
                if (keytable[i]==keypad ) {
                    if (i != 11){
                        if (num*10+i <= 52 && i != 10){
                           num = (num*10) + i ;
                           
                        }
                        else if (i == 10 && ask_num_deal){
                            num = 52/num_players;
                            deal_all = 1;
                        }
                        else {
                            num = 52;
                        }
                        PT_YIELD_TIME_msec(500);
                        pressed = 1;
                    }
                    else{
                        if (ask_num_deal) {
                            num_deal = num;
                            ask_num_deal = 0;
                            ns = 1;
                            num = 0;
                        } else if (ask_players) {
                            ask_num_deal = 1;
                            num_players = num;
                            ask_players = 0;
                            num = 0;
                        }
                    }
                    break;
                }
            }
            // if invalid, two button push, set to -1
            if (i==24) i=-1;
        }
        else i = -1; // no button pushed

        // draw key number
        if (i>-1 && i<10) sprintf(buffer," %d", num);
        if (i==10 ) sprintf(buffer,"  %x *", keypad);
        //if (i==11 ) sprintf(buffer,"  %d #", num);
        //if (i>11 && i<22 ) sprintf(buffer, "  %x shift-%d", keypad, i-12);
        //if (i==22 ) sprintf(buffer,"  %x ahift-*", keypad);
        //if (i==23 ) sprintf(buffer,"  %x shift-#", keypad);
        if (i>-1 && i<12) printLine2(10, buffer, ILI9340_GREEN, ILI9340_BLACK);
        else if (i>-1) printLine2(10, buffer, ILI9340_RED, ILI9340_BLACK);
        
        // NEVER exit while
      } // END WHILE(1)
  PT_END(pt);
} // keypad thread


// === Timer Thread =================================================
// system 1 second interval tick
// prints on TFT and blinks LED on RA0
// thread identifier to set thread parameters
int thread_num_timer;

// update a 1 second tick counter
static PT_THREAD (protothread_timer(struct pt *pt))
{
    PT_BEGIN(pt);
     // set up LED to blink
     mPORTASetBits(BIT_0 );	//Clear bits to ensure light is off.
     mPORTASetPinsDigitalOut(BIT_0 );    //Set port as output
    
      while(1) {
        // yield time 1 second
        PT_YIELD_TIME_msec(1000) ;

        sys_time_seconds++ ;
        // toggle the LED on the big board
        mPORTAToggleBits(BIT_0);
        
        // draw sys_time
//        sprintf(buffer,"Time=%d", sys_time_seconds);
//        printLine2(0, buffer, ILI9340_BLACK, ILI9340_YELLOW);
//        
//        sprintf(buffer,"nd=%d", num_deal);
//        printLine2(2, buffer, ILI9340_BLACK, ILI9340_YELLOW);
//
//        sprintf(buffer,"np=%d", num_players);
//        printLine2(4, buffer, ILI9340_BLACK, ILI9340_YELLOW);
//        
//        sprintf(buffer,"ns=%d", ns);
//        printLine2(6, buffer, ILI9340_BLACK, ILI9340_YELLOW);
//        
//        sprintf(buffer,"deal? #%d", deal_start);
//        printLine2(12, buffer, ILI9340_BLACK, ILI9340_YELLOW);
//        
//        sprintf(buffer,"Card #%d", curr_card);
//        printLine2(14, buffer, ILI9340_BLACK, ILI9340_YELLOW);
        
//        if (restart){
//            tft_setRotation(1);
//            tft_fillRect(0, 0, 320, 240, ILI9340_BLACK);// x,y,w,h,radius,color
//            tft_setCursor(15, 100);
//            tft_setTextColor(ILI9340_YELLOW); 
//            tft_setTextSize(2.5);
//            sprintf(buffer,"DEAL DONE");
//        }
        
        // NEVER exit while
      } // END WHILE(1)
  PT_END(pt);
} // timer thread


//=== State terminal thread =============================================
static PT_THREAD (protothread_state(struct pt *pt)) {
    PT_BEGIN(pt);
    
    while (1) {
        PT_YIELD_TIME_msec(1000);
        tft_fillRect(0, 0, 240, 320, ILI9340_BLACK);// x,y,w,h,radius,color
        tft_setRotation(1);
        tft_setCursor(15, 110);
        tft_setTextColor(ILI9340_YELLOW); 
        tft_setTextSize(2); 
        sprintf(buffer,"HOW MANY PLAYERS?");
        tft_writeString(buffer);
        sprintf(buffer," Type number then press #");
        printLine2(7, buffer, ILI9340_YELLOW, ILI9340_BLACK);
        PT_YIELD_UNTIL(pt, ask_num_deal);
        tft_fillRect(0, 0, 320, 240, ILI9340_BLACK);// x,y,w,h,radius,color
        tft_setCursor(15, 110);
        tft_setTextColor(ILI9340_YELLOW); 
        tft_setTextSize(2);
        sprintf(buffer,"HOW MANY CARDS?");
        tft_writeString(buffer);
        sprintf(buffer," Type number then press #");
        printLine2(7, buffer, ILI9340_YELLOW, ILI9340_BLACK);  
        sprintf(buffer," Press * then # to deal all");
        printLine2(8, buffer, ILI9340_YELLOW, ILI9340_BLACK);
        PT_YIELD_TIME_msec(500);
        pressed = 1;
        PT_YIELD_UNTIL(pt, !ask_num_deal && ns ==1);
        tft_fillRect(0, 0, 320, 240, ILI9340_BLACK);// x,y,w,h,radius,color
        tft_setCursor(15, 110);
        tft_setTextColor(ILI9340_YELLOW); 
        tft_setTextSize(2);
        sprintf(buffer,"SHUFFLING...");
        tft_writeString(buffer);

        PT_YIELD_UNTIL(pt, deal_start);
        ns = 0;
        tft_fillRect(0, 0, 320, 240, ILI9340_BLACK);// x,y,w,h,radius,color
        tft_setCursor(15, 110);
        tft_setTextColor(ILI9340_YELLOW); 
        tft_setTextSize(2);
        sprintf(buffer,"DEALING...");
        tft_writeString(buffer);
//        sprintf(buffer,"np=%d", num_players);
//        printLine2(4, buffer, ILI9340_BLACK, ILI9340_YELLOW);        
//        sprintf(buffer,"nd=%d", num_deal);
//        printLine2(2, buffer, ILI9340_BLACK, ILI9340_YELLOW);
//        sprintf(buffer,"Card #%d", curr_card);
//        printLine2(14, buffer, ILI9340_BLACK, ILI9340_YELLOW);
//        sprintf(buffer,"total cards #%d", total_cards);
//        printLine2(10, buffer, ILI9340_BLACK, ILI9340_YELLOW);
//        sprintf(buffer,"all? #%d", deal_all);
//        printLine2(3, buffer, ILI9340_BLACK, ILI9340_YELLOW);
//        sprintf(buffer,"curr_card=%d", curr_card);
//        printLine2(1, buffer, ILI9340_BLACK, ILI9340_YELLOW);
        PT_YIELD_UNTIL(pt, restart); 
        tft_fillRect(0, 0, 320, 240, ILI9340_BLACK);// x,y,w,h,radius,color
        tft_setCursor(15, 110);
        tft_setTextColor(ILI9340_YELLOW); 
        tft_setTextSize(2);
        sprintf(buffer,"DEAL DONE");        

        
        pressed = 1; 
        num = 0;
        num_players = 0;
        ask_num_deal = 0;
        num_deal = 0;
        curr_player = 0;
        //curr_card = 0;
        ask_players = 1;
        restart = 0;
        deal_all = 0;
        memset(player_arr, 0, sizeof(player_arr));
        PT_YIELD_TIME_msec(3000);
    }
    
    PT_END(pt);
}

//=== Shuffle mode terminal thread =============================================
static PT_THREAD (protothread_shuffle(struct pt *pt)) {
    PT_BEGIN(pt);
    
    mPORTASetBits(BIT_2 );	//Clear bits to ensure light is off.
    mPORTASetPinsDigitalOut(BIT_2 );    //Set port as output

    mPORTASetBits(BIT_3 );	//Clear bits to ensure light is off.
    mPORTASetPinsDigitalOut(BIT_3 );    //Set port as output
         
    while(1) {
        
//        r_dutycycle = dc_CCW;
//        PT_YIELD_TIME_msec(time_for_360_CCW);
//        r_dutycycle = 0;
//        PT_YIELD_TIME_msec(1000);
//        r_dutycycle = dc_CW;
//        PT_YIELD_TIME_msec(time_for_360_CW);
//        r_dutycycle = 0;
        
        // shuffle in
        // yield thread while deal mode is on
        PT_YIELD_UNTIL(pt, !deal_start);
        // yield thread while we don't want to shuffle
        PT_YIELD_UNTIL(pt, (ns == 1));
        dutycycle = s_dc;
        // time for each shuffle in
        PT_YIELD_TIME_msec(time_for_half);
        dutycycle = 0;
        
//        if (ns > 1) {        
//            // lift cards w servos
//
//            // shuffle out
//            // set BIT_3 high
//            mPORTAToggleBits(BIT_3);
//            // wait 5 seconds
//            PT_YIELD_TIME_msec(time_for_sout);
//            // set BIT_2 high, BIT_3 low
//            mPORTAToggleBits(BIT_2);
//            mPORTAToggleBits(BIT_3);
//            // wait 5 seconds
//            PT_YIELD_TIME_msec(time_for_sout);
//            // set both low
//            mPORTASetBits(BIT_2 );	//Clear bits to ensure light is off.
//            mPORTASetBits(BIT_3 );	//Clear bits to ensure light is off.
//        }
        
        // update shuffle counter
        ns--;
        
        // after shuffling is over, deal mode is on
        //if (ns == 0){
        deal_start = 1;
        //}
    }
    
    PT_END(pt);
}


//=== Deal mode terminal thread ================================================

static PT_THREAD (protothread_deal(struct pt *pt)) {
    PT_BEGIN(pt);
    //int total_cards = 4;
    while(1) {
        // yield thread until deal mode is on
        PT_YIELD_UNTIL(pt, deal_start);
        

        
        PT_YIELD_TIME_msec(500);
        
        // fancy random code for dealing and rotating
        int time_rotate = 0;      
        // pick a new player
        prev_player = curr_player;
        curr_player = rand() % (num_players-1);

        // check if finished dealing to that player
        if(deal_all){
            while(player_arr[curr_player] >= num_deal+1){
                if (curr_player >= num_players-1) {
                    curr_player = 0;
                } else {
                    curr_player++;
                }
            }   
        }
        else{
            while(player_arr[curr_player] >= num_deal){
                if (curr_player >= num_players-1) {
                    curr_player = 0;
                } else {
                    curr_player++;
                }
            }
        }

        int diff = curr_player - prev_player;
        if (diff < 0) {
            r_dutycycle = dc_CCW; // CCW
            time_rotate = (-diff) * (time_for_360_CCW/num_players);
        } else {
            r_dutycycle = dc_CW; // CW
            time_rotate = diff * (time_for_360_CW/num_players);
        }

        PT_YIELD_TIME_msec(time_rotate);
        r_dutycycle = 0;

        PT_YIELD_TIME_msec(500);

//        if (curr_card < 5) {
//            deal_dc = 21000;
//        } else if (total_cards - curr_card == 10) {
//            deal_dc = 5000;
//        } else if (total_cards - curr_card == 15) {
//            deal_dc = 9000;
//        } else if (total_cards - curr_card == 25) {
//            deal_dc = 14000;
//        } else {
//            deal_dc -= 250;
//        }
        
        if (curr_card < 5) {
            deal_dc = 25500;
        } else if (curr_card < 10){
            deal_dc = 19800;
        } else if (curr_card == 42) {
            deal_dc = 7590;
        } else if (curr_card == 37) {
            deal_dc = 10590;
        } else if (curr_card == 27) {
            deal_dc = 14590;
        } else if (curr_card == 17){
            deal_dc = 17800;
        }else {
            deal_dc -= 175;
        }
        
        // deal - nudge
//        if (curr_card%7 != 0) {
            d_dutycycle = 23800;
            PT_YIELD_TIME_msec(time_for_one/2.1);
            d_dutycycle = 0;
            PT_YIELD_TIME_msec(500);
            d_dutycycle = deal_dc;  
//        } else {
//            d_dutycycle = deal_dc;
//        }
        
        d2_dutycycle = deal_dc/4;
        PT_YIELD_TIME_msec(time_for_one);
        d_dutycycle = 0;
        PT_YIELD_TIME_msec(time_for_one/1.5);
        d2_dutycycle = 0;

        player_arr[curr_player] += 1;
        curr_card++;

        PT_YIELD_TIME_msec(500);
        
        // check end deal
        if(deal_all && curr_card >= 52){
            restart = 1;
            deal_start = 0;

            pressed = 1;
            // back to starting position
            time_rotate = (curr_player * (time_for_360_CCW/num_players));
            r_dutycycle = dc_CCW;
            PT_YIELD_TIME_msec(time_rotate);
            r_dutycycle = 0;
            curr_card = 0;
        }
        if(!deal_all && curr_card >= total_cards){
            // halfway between player 1 and player 2
            time_rotate = (curr_player * (time_for_360_CCW/num_players) - (time_for_360_CCW/num_players)/2);
            r_dutycycle = dc_CCW;
            PT_YIELD_TIME_msec(time_rotate);
            r_dutycycle = 0;
            
            PT_YIELD_TIME_msec(500);
            
            // deal out rest of cards
            d_dutycycle = dc;
            d2_dutycycle = dc/15;
            PT_YIELD_TIME_msec(time_for_one*(52-curr_card/2));
            d_dutycycle = 0;
            d2_dutycycle = 0;
            // back to starting position
            //time_rotate = (curr_player * (time_for_360_CCW/num_players));
            time_rotate = (time_for_360_CCW/num_players)/2;
            r_dutycycle = dc_CCW;
            PT_YIELD_TIME_msec(time_rotate*1.2);
            r_dutycycle = 0;
            
            pressed = 1;            
            curr_card = 0;
            deal_start = 0;
            restart = 1;

        }

        
        sprintf(buffer," Curr Card=%d", curr_card);
        printLine2(7, buffer, ILI9340_YELLOW, ILI9340_BLACK);
        int pc = player_arr[curr_player];
        sprintf(buffer," Player %d has %d Cards", curr_player+1, pc);
        printLine2(8, buffer, ILI9340_BLACK, ILI9340_BLACK);
        printLine2(8, buffer, ILI9340_YELLOW, ILI9340_BLACK);
        sprintf(buffer," Whole Deck?=%d", deal_all);
        printLine2(9, buffer, ILI9340_YELLOW, ILI9340_BLACK);     
    }
    
    PT_END(pt);
}

// === Main  ======================================================

void main(void) {
  //=====Setup for ADC===========////
    PPSInput (2, U2RX, RPA1);
    
    // === Config timer and output compare to make PWM ========
    // set up timer2 to generate the wave period -- SET this to 1 mSec!
    OpenTimer2(T2_ON | T2_SOURCE_INT | T2_PS_1_1, 40000);
    // Need ISR to compute PID controller
    ConfigIntTimer2(T2_INT_ON | T2_INT_PRIOR_2);
    mT2ClearIntFlag(); // and clear the interrupt flag
    // set up compare3 for PWM mode
    OpenOC1(OC_ON | OC_TIMER2_SRC | OC_PWM_FAULT_PIN_DISABLE , dutycycle, dutycycle); //
    // OC3 is PPS group 4, map to RPB9 (pin 18)
    PPSOutput(1, RPB7, OC1);
    
    // === Config timer and output compare to make PWM ========
    // set up timer2 to generate the wave period -- SET this to 1 mSec!
    OpenTimer2(T2_ON | T2_SOURCE_INT | T2_PS_1_1, 40000);
    // Need ISR to compute PID controller
    ConfigIntTimer2(T2_INT_ON | T2_INT_PRIOR_2);
    mT2ClearIntFlag(); // and clear the interrupt flag
    // set up compare3 for PWM mode
    OpenOC2(OC_ON | OC_TIMER2_SRC | OC_PWM_FAULT_PIN_DISABLE , d_dutycycle, d_dutycycle); //
    // OC3 is PPS group 4, map to RPB9 (pin 18)
    PPSOutput(2, RPA1, OC2);
   
    
    OpenTimer2(T2_ON | T2_SOURCE_INT | T2_PS_1_1, 40000);
    // Need ISR to compute PID controller
    ConfigIntTimer2(T2_INT_ON | T2_INT_PRIOR_2);
    mT2ClearIntFlag(); // and clear the interrupt flag
    // set up compare3 for PWM mode
    OpenOC3(OC_ON | OC_TIMER2_SRC | OC_PWM_FAULT_PIN_DISABLE , d2_dutycycle, d2_dutycycle); //
    // OC3 is PPS group 4, map to RPB9 (pin 18)
    PPSOutput(4, RPA3, OC3);
    
    
    OpenTimer2(T2_ON | T2_SOURCE_INT | T2_PS_1_1, 40000);
    // Need ISR to compute PID controller
    ConfigIntTimer2(T2_INT_ON | T2_INT_PRIOR_2);
    mT2ClearIntFlag(); // and clear the interrupt flag
    // set up compare3 for PWM mode
    OpenOC4(OC_ON | OC_TIMER2_SRC | OC_PWM_FAULT_PIN_DISABLE , r_dutycycle, r_dutycycle); //
    // OC3 is PPS group 4, map to RPB9 (pin 18)
    PPSOutput(3, RPB13, OC4);

//    // === SPI channel for DAC and Expander ============
//    // control CS for DAC
//    mPORTBSetPinsDigitalOut(BIT_4);
//    mPORTBSetBits(BIT_4);
//    // SCK2 is on pin RB15 
//    // SDO2 (MOSI) is in PPS output group 2, and could be connected to RB5 which is pin 14
//    PPSOutput(3, RPA8, SDO2);
//    // 16 bit transfer CKP=1 CKE=1
//    // possibles SPI_OPEN_CKP_HIGH;   SPI_OPEN_SMP_END;  SPI_OPEN_CKE_REV
//    // For any given peripherial, you will need to match these
//    // NOTE!! IF you are using the port expander THEN
//    // >>> clk divider must be set to 4 for 10 MHz <<<
//    SpiChnOpen(SPI_CHANNEL2, SPI_OPEN_ON | SPI_OPEN_MODE16 | SPI_OPEN_MSTEN | SPI_OPEN_CKE_REV , 4);
//  // end SPI setup
    
//  // === build the DDS sine lookup table =======
//   // scaled to produce values between 0 and 4096
//   int ii;
//   for (ii = 0; ii < sine_table_size; ii++){
//         sin_table[ii] = (int)(2047*sin((float)ii*6.283/(float)sine_table_size));
//    }

  // === setup system wide interrupts  ========
  INTEnableSystemMultiVectoredInt();
  
  // === TFT setup ============================
  // init the display in main since more than one thread uses it.
  // NOTE that this init assumes SPI channel 1 connections
  tft_init_hw();
  tft_begin();
  tft_fillScreen(ILI9340_BLACK);
  //240x320 vertical display
  tft_setRotation(0); // Use tft_setRotation(1) for 320x240
  
  
   // the ADC ///////////////////////////////////////
        // configure and enable the ADC
	CloseADC10();	// ensure the ADC is off before setting the configuration

	// define setup parameters for OpenADC10
	// Turn module on | ouput in integer | trigger mode auto | enable autosample
    // ADC_CLK_AUTO -- Internal counter ends sampling and starts conversion (Auto convert)
    // ADC_AUTO_SAMPLING_ON -- Sampling begins immediately after last conversion completes; SAMP bit is automatically set
    // ADC_AUTO_SAMPLING_OFF -- Sampling begins with AcquireADC10();
    #define PARAM1  ADC_FORMAT_INTG16 | ADC_CLK_AUTO | ADC_AUTO_SAMPLING_OFF //

	// define setup parameters for OpenADC10
	// ADC ref external  | disable offset test | disable scan mode | do 1 sample | use single buf | alternate mode off
	#define PARAM2  ADC_VREF_AVDD_AVSS | ADC_OFFSET_CAL_DISABLE | ADC_SCAN_OFF | ADC_SAMPLES_PER_INT_1 | ADC_ALT_BUF_OFF | ADC_ALT_INPUT_OFF
    //
	// Define setup parameters for OpenADC10
    // use peripherial bus clock | set sample time | set ADC clock divider
    // ADC_CONV_CLK_Tcy2 means divide CLK_PB by 2 (max speed)
    // ADC_SAMPLE_TIME_5 seems to work with a source resistance < 1kohm
    #define PARAM3 ADC_CONV_CLK_PB | ADC_SAMPLE_TIME_5 | ADC_CONV_CLK_Tcy2 //ADC_SAMPLE_TIME_15| ADC_CONV_CLK_Tcy2

	// define setup parameters for OpenADC10
	// set AN11 and  as analog inputs
	#define PARAM4	ENABLE_AN11_ANA // pin 24

	// define setup parameters for OpenADC10
	// do not assign channels to scan
	#define PARAM5	SKIP_SCAN_ALL

	// use ground as neg ref for A | use AN11 for input A     
	// configure to sample AN11 
	SetChanADC10( ADC_CH0_NEG_SAMPLEA_NVREF | ADC_CH0_POS_SAMPLEA_AN11 ); // configure to sample AN11 
	OpenADC10( PARAM1, PARAM2, PARAM3, PARAM4, PARAM5 ); // configure ADC using the parameters defined above

	EnableADC10(); // Enable the ADC
    ///////////////////////////////////////////////////////
  
  
  // === config threads ========================
  PT_setup();
  //PT_INIT(&pt_key);
  
  // === identify the threads to the scheduler =====
  // add the thread function pointers to be scheduled
  // --- Two parameters: function_name and rate. ---
  // rate=0 fastest, rate=1 half, rate=2 quarter, rate=3 eighth, rate=4 sixteenth,
  // rate=5 or greater DISABLE thread!
  // If you need to access specific thread descriptors (to change rate), 
  // then return the list index
  thread_num_timer = pt_add(protothread_timer, 0);
  pt_add(protothread_shuffle, 0);
  pt_add(protothread_key, 1);
  pt_add(protothread_state, 0);
  pt_add(protothread_deal, 0);

  // === initalize the scheduler ====================
  PT_INIT(&pt_sched) ;
  
  
  // >>> CHOOSE the scheduler method: <<<
  // (1)
  // SCHED_ROUND_ROBIN just cycles thru all defined threads
  //pt_sched_method = SCHED_ROUND_ROBIN ;
  
  // (2)
  // SCHED_RATE executes some threads more often then others
  // -- rate=0 fastest, rate=1 half, rate=2 quarter, rate=3 eighth, rate=4 sixteenth,
  // -- rate=5 or greater DISABLE thread!
  // pt_sched_method = SCHED_RATE ;
  
  pt_sched_method = SCHED_ROUND_ROBIN ;
  
  // === scheduler thread =======================
  // scheduler never exits
  PT_SCHEDULE(protothread_sched(&pt_sched));
 
  // ============================================
  
  
} // main

// === end  ======================================================